/*global angular*/
/*jslint node: true */
/*jslint plusplus: true*/

'use strict';

var app = angular.module('readerApp', ['ui.bootstrap', 'ngCookies']);

app.directive('imageonload', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            element.bind('load', function () {
                //call the function that was passed
                scope.$apply(attrs.imageonload);
            });
        }
    };
});

app.controller('readerController', function ($scope, $location, $window, $document, $cookies) {

    $scope.loadPage = function () {
        $scope.isLoading = true;
        $scope.pageSource = "/" + $scope.documentType + "reader/" + $scope.documentId + "?page=" + $scope.currPageNb + "&width=" + $scope.IMAGE_MAX_NATIV_WIDTH;

        // set cookie
        var expireDate = new Date();
        expireDate.setFullYear(expireDate.getFullYear() + 1);
        $cookies.put($scope.documentType + $scope.documentId, $scope.currPageNb, {
            'expires': expireDate
        });
    };

    $scope.nextPage = function () {
        $scope.currentWay = $scope.WAY.FORWARD;
        if (($scope.prevPageType === $scope.PAGETYPE.LEFT && $scope.pageSplit === $scope.SPLIT.L2R) || ($scope.prevPageType === $scope.PAGETYPE.RIGHT && $scope.pageSplit === $scope.SPLIT.R2L)) {
            $scope.prevPageType = $scope.applyPageSplit();
        } else {
            if ($scope.currPageNb < $scope.nbPages - 1) {
                $scope.currPageNb = $scope.currPageNb + 1;
                $scope.loadPage();
            }
        }

    };

    $scope.previousPage = function () {
        $scope.currentWay = $scope.WAY.BACKWARD;
        if (($scope.prevPageType === $scope.PAGETYPE.RIGHT && $scope.pageSplit === $scope.SPLIT.L2R) || ($scope.prevPageType === $scope.PAGETYPE.LEFT && $scope.pageSplit === $scope.SPLIT.R2L)) {
            $scope.prevPageType = $scope.applyPageSplit();
        } else {
            if ($scope.currPageNb > 0) {
                $scope.currPageNb = $scope.currPageNb - 1;
                $scope.loadPage();
            }
        }
    };

    $scope.toggleMenu = function () {
        $scope.displayMenu = !$scope.displayMenu;
    };

    $scope.imgLoaded = function () {
        // hide loading spinner
        $scope.isLoading = false;
        $scope.prevPageType = $scope.PAGETYPE.SINGLE;
        $scope.prevPageType = $scope.applyPageSplit();
        $window.scrollTo(0, 0);
        $scope.preloadNextPages();
    };

    $scope.goToPage = function () {
        var goToNb = $window.prompt('Choose page number', '');
        if (goToNb !== null) {
            goToNb = parseInt(goToNb, 10);
            if ((goToNb <= $scope.nbPages) && (goToNb > 0) && (goToNb - 1 !== $scope.currPageNb)) {
                $scope.currentWay = $scope.WAY.FORWARD;
                $scope.currPageNb = goToNb - 1;
                $scope.loadPage();
            }
        }
    };

    $scope.handleKey = function (event) {
        var charCode = event.which || event.keyCode;
        if (charCode === 37) {
            // left arrow
            $scope.previousPage();
        } else if (charCode === 39) {
            // right arrow
            $scope.nextPage();
        } else if (charCode === 27) {
            // escape
            history.back();
        } else if (charCode === 77) {
            // 'm'
            $scope.displayMenu = !$scope.displayMenu;
        } else if (charCode === 71) {
            // 'g'
            $scope.goToPage();
        } else if (charCode === 32) {
            // space
            if ($window.pageYOffset + $window.innerHeight >= $scope.getDocHeight()) {
                $scope.nextPage();
            }
        }
    };

    $scope.getDocHeight = function () {
        var dcmt = document;
        return Math.max(
            dcmt.body.scrollHeight,
            dcmt.documentElement.scrollHeight,
            dcmt.body.offsetHeight,
            dcmt.documentElement.offsetHeight,
            dcmt.body.clientHeight,
            dcmt.documentElement.clientHeight
        );
    };

    $scope.updatePageSize = function () {
        switch ($scope.pageSize) {
        case 0:
            $scope.pageStyle = "fillWidth";
            break;
        case 1:
            $scope.pageStyle = "fillHeight";
            break;
        case 2:
            $scope.pageStyle = "original";
            break;
        }
        // save user choice as pageStyle might be modified when splitting double pages
        $scope.userPageStyle = $scope.pageStyle;
        $scope.applyPageSplit();
    };

    $scope.applyPageSplit = function () {
        var nativeImg = new Image(),
            img = document.getElementById('pagecontent'),
            newHeight = 0,
            newWidth = 0,
            leftOffset = 0,
            topOffset = 0,
            showLeftPage = true,
            newPageType;
        nativeImg.src = img.src;

        //alert("Width: " + nativeImg.width + " - Height: " + nativeImg.height);
        //alert(img.width);

        if (nativeImg.width > nativeImg.height && $scope.pageSplit !== 2) { // '2' is 'no split'
            $scope.pageStyle = "";

            // determine which page to show (left or right)
            if ($scope.prevPageType === $scope.PAGETYPE.SINGLE) {
                if ($scope.currentWay === $scope.WAY.FORWARD) {
                    showLeftPage = $scope.pageSplit === $scope.SPLIT.L2R;
                } else {
                    showLeftPage = $scope.pageSplit === $scope.SPLIT.R2L;
                }
            } else {
                if ($scope.prevPageType === $scope.PAGETYPE.LEFT) {
                    if ($scope.currentWay === $scope.WAY.FORWARD) {
                        showLeftPage = $scope.pageSplit === $scope.SPLIT.R2L;
                    } else {
                        showLeftPage = $scope.pageSplit === $scope.SPLIT.L2R;
                    }
                } else {
                    if ($scope.currentWay === $scope.WAY.FORWARD) {
                        showLeftPage = $scope.pageSplit === $scope.SPLIT.R2L;
                    } else {
                        showLeftPage = $scope.pageSplit === $scope.SPLIT.L2R;
                    }
                }
            }

            if (showLeftPage) {
                // show left page
                switch ($scope.pageSize) {
                case 0: // fill width
                    newWidth = $window.innerWidth - 20; // -20 to remove the width of the scroll bar, which is about 17px large    
                    newHeight = newWidth * 2 * nativeImg.height / nativeImg.width;
                    if (newHeight < $window.innerHeight) {
                        topOffset = ($window.innerHeight - newHeight) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': (newWidth * 2) + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth + 'px, ' + newHeight + 'px, 0px)',
                        'top': topOffset + 'px'
                    };
                    break;
                case 1: // fill height
                    newHeight = $window.innerHeight - 20; // -20 to remove the width of the scroll bar, which is about 17px large    
                    newWidth = $window.innerHeight * nativeImg.width / nativeImg.height;
                    if (newWidth / 2 < $window.innerWidth) {
                        leftOffset = ($window.innerWidth - newWidth / 2) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': newWidth + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth / 2 + 'px, ' + newHeight + 'px, 0px)',
                        'left': leftOffset + 'px'
                    };
                    break;
                case 2: // original
                    newHeight = nativeImg.height;
                    newWidth = nativeImg.width;
                    if (newHeight < $window.innerHeight) {
                        topOffset = ($window.innerHeight - newHeight) / 2;
                    }
                    if (newWidth / 2 < $window.innerWidth) {
                        leftOffset = ($window.innerWidth - newWidth / 2) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': newWidth + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth / 2 + 'px, ' + newHeight + 'px, 0px)',
                        'top': topOffset + 'px',
                        'left': leftOffset + 'px'
                    };
                    break;
                }
                newPageType = $scope.PAGETYPE.LEFT;
            } else {
                // show right page
                switch ($scope.pageSize) {
                case 0: // fill width
                    newWidth = $window.innerWidth - 20; // -20 to remove the width of the scroll bar, which is about 17px large    
                    newHeight = newWidth * 2 * nativeImg.height / nativeImg.width;
                    if (newHeight < $window.innerHeight) {
                        topOffset = ($window.innerHeight - newHeight) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': (newWidth * 2) + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth * 2 + 'px, ' + newHeight + 'px, ' + newWidth + 'px)',
                        'top': topOffset + 'px',
                        'left': -newWidth + 'px'
                    };
                    break;
                case 1: // fill height
                    newHeight = $window.innerHeight - 20; // -20 to remove the width of the scroll bar, which is about 17px large    
                    newWidth = $window.innerHeight * nativeImg.width / nativeImg.height;
                    if (newWidth / 2 < $window.innerWidth) {
                        leftOffset = ($window.innerWidth - newWidth / 2) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': newWidth + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth + 'px, ' + newHeight + 'px, ' + newWidth / 2 + 'px)',
                        'left': leftOffset - newWidth / 2 + 'px'
                    };
                    break;
                case 2: // original
                    newHeight = nativeImg.height;
                    newWidth = nativeImg.width;
                    if (newHeight < $window.innerHeight) {
                        topOffset = ($window.innerHeight - newHeight) / 2;
                    }
                    if (newWidth / 2 < $window.innerWidth) {
                        leftOffset = ($window.innerWidth - newWidth / 2) / 2;
                    }
                    $scope.pageSplitStyle = {
                        'position': 'absolute',
                        'width': newWidth + 'px',
                        'height': newHeight + 'px',
                        'clip': 'rect(0px,' + newWidth + 'px, ' + newHeight + 'px, ' + newWidth / 2 + 'px)',
                        'top': topOffset + 'px',
                        'left': leftOffset - newWidth / 2 + 'px'
                    };
                    break;
                }
                newPageType = $scope.PAGETYPE.RIGHT;
            }

        } else {
            // restore user chosen style            
            $scope.pageSplitStyle = "";
            $scope.pageStyle = $scope.userPageStyle;
            newPageType = $scope.PAGETYPE.SINGLE;
        }
        return newPageType;
    };

    $scope.preloadNextPages = function () {
        var p = 0,
            i = 0,
            images = [];
        for (i = 0; i < $scope.PRELOAD_IMG_NB; i++) {
            p = $scope.currPageNb + i + 1;
            if (p < $scope.nbPages - 1) {
                images[i] = new Image();
                images[i].src = "/" + $scope.documentType + "reader/" + $scope.documentId + "?page=" + p + "&width=" + $scope.IMAGE_MAX_NATIV_WIDTH;
            } else {
                break;
            }
        }
    };

    $scope.writeReaderSettings = function () {
        var expireDate = new Date();
        expireDate.setFullYear(expireDate.getFullYear() + 1);
        $cookies.put($scope.READER_SETTINGS, $scope.pageSize + '#' + $scope.pageSplit, {
            'expires': expireDate
        });
    };


    ////////////////////
    // INITIALIZATION //
    ////////////////////

    // a few constants
    $scope.SPLIT = {
        L2R: 0,
        R2L: 1,
        NONE: 2
    };
    $scope.PAGETYPE = {
        SINGLE: 0,
        LEFT: 1,
        RIGHT: 2
    };
    $scope.WAY = {
        FORWARD: 0,
        BACKWARD: 1
    };
    $scope.READER_SETTINGS = 'ComicReaderSettings';
    $scope.PRELOAD_IMG_NB = 2;
    $scope.IMAGE_MAX_NATIV_WIDTH = 1280;


    // get URL parameters
    var pageFromCookie, readerSettings, readerSettingsArray, searchObject = $location.search();
    $scope.documentId = searchObject.docId;
    $scope.documentType = searchObject.type;
    $scope.currPageNb = parseInt(searchObject.startIndex, 10);
    $scope.nbPages = parseInt(searchObject.nbPages, 10);

    // default values
    $scope.displayMenu = false;
    $scope.displayLoadingMsg = false;
    $scope.isLoading = false;
    $scope.displayGoToField = true;
    $scope.pageSource = "";
    $scope.pageSize = 0;
    $scope.pageStyle = "fillWidth";
    $scope.userPageStyle = $scope.pageStyle;
    $scope.pageSplit = $scope.SPLIT.L2R;
    $scope.currentWay = $scope.WAY.FORWARD;
    $scope.prevPageType = $scope.PAGETYPE.SINGLE;

    // get cookies parameters
    pageFromCookie = $cookies.get($scope.documentType + $scope.documentId);
    if (pageFromCookie) {
        $scope.currPageNb = parseInt(pageFromCookie, 10);
    }
    readerSettings = $cookies.get($scope.READER_SETTINGS);
    if (readerSettings) {
        readerSettingsArray = readerSettings.split('#');
        $scope.pageSize = parseInt(readerSettingsArray.shift(), 10);
        $scope.pageSplit = parseInt(readerSettingsArray.shift(), 10);
    }

    // add listeners
    $scope.$watch('pageSize', $scope.updatePageSize);
    $scope.$watch('pageSize', $scope.writeReaderSettings);
    $scope.$watch('pageSplit', $scope.applyPageSplit);
    $scope.$watch('pageSplit', $scope.writeReaderSettings);

    // let's go
    $scope.loadPage();

    // END INITIALIZATION
});